@@ -215,11 +215,23 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
   exif_profile=AcquireStringInfo(length);
   error=heif_image_handle_get_metadata(image_handle,id,
     GetStringInfoDatum(exif_profile));
-  if ((IsHEIFSuccess(image,&error,exception) != MagickFalse) &&
-      (length > 4))
+  if ((IsHEIFSuccess(image,&error,exception) != MagickFalse) && (length > 4))
     {
-      (void) DestroyStringInfo(SplitStringInfo(exif_profile,4));
-      (void) SetImageProfile(image,"exif",exif_profile,exception);
+      /*
+        Extract Exif profile.
+      */
+      StringInfo *snippet = SplitStringInfo(exif_profile,4);
+      unsigned int offset = 0;
+      offset|=(unsigned int) (*(GetStringInfoDatum(snippet)+0)) << 24;
+      offset|=(unsigned int) (*(GetStringInfoDatum(snippet)+1)) << 16;
+      offset|=(unsigned int) (*(GetStringInfoDatum(snippet)+2)) << 8;
+      offset|=(unsigned int) (*(GetStringInfoDatum(snippet)+3)) << 0;
+      snippet=DestroyStringInfo(snippet);
+      if (offset < GetStringInfoLength(exif_profile))
+        {
+          (void) DestroyStringInfo(SplitStringInfo(exif_profile,offset));
+          (void) SetImageProfile(image,"exif",exif_profile,exception);
+        }
     }
   exif_profile=DestroyStringInfo(exif_profile);
   return(MagickTrue);
@@ -844,18 +856,8 @@ static void WriteProfile(struct heif_context *context,Image *image,
     profile=GetImageProfile(image,name);
     length=GetStringInfoLength(profile);
     if (LocaleCompare(name,"EXIF") == 0)
-      {
-        length=GetStringInfoLength(profile);
-        if (length > 65533L)
-          {
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              CoderWarning,"ExifProfileSizeExceedsLimit","`%s'",
-              image->filename);
-            length=65533L;
-          }
-        (void) heif_context_add_exif_metadata(context,image_handle,
-          (void*) GetStringInfoDatum(profile),(int) length);
-      }
+      (void) heif_context_add_exif_metadata(context,image_handle,
+        (void*) GetStringInfoDatum(profile),(int) length);
     if (LocaleCompare(name,"XMP") == 0)
       for (i=0; i < (ssize_t) GetStringInfoLength(profile); i+=65533L)
       {
@@ -1199,6 +1201,9 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     MagickBooleanType
       lossless = image_info->quality >= 100 ? MagickTrue : MagickFalse;
 
+    struct heif_encoding_options
+      *options;
+
     /*
       Get encoder for the specified format.
     */
@@ -1300,10 +1305,15 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
               break;
           }
       }
+#endif
+    options=heif_encoding_options_alloc();
+#if LIBHEIF_NUMERIC_VERSION >= 0x010e0000
+    if (image->orientation != UndefinedOrientation)
+      options->image_orientation=(enum heif_orientation) image->orientation;
 #endif
     error=heif_context_encode_image(heif_context,heif_image,heif_encoder,
-      (const struct heif_encoding_options *) NULL,
-      (struct heif_image_handle **) NULL);
+      options,(struct heif_image_handle **) NULL);
+    heif_encoding_options_free(options);
     if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
       break;
     status=IsHEIFSuccess(image,&error,exception);
